/* sanebuttond -- SANE button monitoring daemon
   Using the SANE library.
   Copyright (C) 2004 Rene Rebe

   You can contact me at <rene@exactcode.de>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program;a if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Compile with:
   g++ `sane-config --libs -cflags` sanebuttonsd.cc  -o sanebuttonsd */

#include <assert.h>
#include <getopt.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#include <sys/types.h>
#include <sys/stat.h>

extern "C" {
#include <sane/sane.h>
}

#include <string>
#include <vector>
#include <iostream>

static SANE_Handle device;

int
main (int argc, char **argv)
{
  const SANE_Option_Descriptor *opt;
  const SANE_Device **device_list;
  SANE_Int num_dev_options = 0;

  const char *devname = 0;
  const char *defdevname = 0;
  
  SANE_Status status;
  char *full_optstring;
  SANE_Int version_code;

  /* simple argument parsing ... */
  char* script = 0;
  for (argv++; *argv; argv++)
  {
    if (!strcmp(*argv, "--script")) {
      argv++;
      if (*argv) {
	script = *argv;
      }
      else {
	std::cout << "--script needs an argument!" << std::endl;
	exit (1);
      }
    }
    else {
      std::cout << "unknown option: " << *argv << std::endl;
      exit (1);
    }
  }

  sane_init (&version_code, 0);
  if (!devname)
    {
      /* If no device name was specified explicitly, we look at the
         environment variable SANE_DEFAULT_DEVICE.  If this variable
         is not set, we open the first device we find (if any): */
      devname = getenv ("SANE_DEFAULT_DEVICE");
      if (!devname)
	{
	  std::cout << "Scanning for devices ..." << std::endl;
	  status = sane_get_devices (&device_list, SANE_FALSE);
	  if (status != SANE_STATUS_GOOD) {
	    std::cerr << "sane_get_devices() failed: "
		      <<  sane_strstatus (status) << std::endl;
	    return (1);
	  }
	  if (!device_list[0]) {
	    std::cerr << "no SANE devices found!" << std::endl;
	    return (1);
	  }

          std::cerr << device_list[0]->name << " " << device_list[0]->vendor << " "
	            << device_list[0]->model << " " << device_list[0]->type << std::endl;

	  devname = device_list[0]->name;
	}
    }
  
  // now we should have a valid devname
  
  status = sane_open (devname, &device);
  if (status != SANE_STATUS_GOOD) {
    std::cerr << "open of device " << devname << " failed: "
	      << sane_strstatus (status) << std::endl;
    
    device = 0;
    return (1);
  }
  
  if (!device) {
    std::cerr << "No SANE device detected!"
	      << sane_strstatus (status) << std::endl;
    
    return (1);
  }
 
  /* We got a device, find out how many options it has: */
  status = sane_control_option (device, 0, SANE_ACTION_GET_VALUE,
				&num_dev_options, 0);
  if (status != SANE_STATUS_GOOD) {
    std::cerr << "unable to determine option count" << std::endl;
    return (1);
  }
  
  bool button_pressed = false;
  std::vector<std::string> args;
  while (script && !button_pressed || !script)
    {
      if (!script) /* if no script we run in demo mode */
	button_pressed = false;
      for (int i = 0; i < num_dev_options; ++i)
	{
	  opt = 0;
	  opt = sane_get_option_descriptor (device, i);
	  
	  if (opt->name && SANE_OPTION_IS_ACTIVE(opt->cap)) {
	    std::string optname (opt->name);
	    if (optname.find("button") == 0 || optname == "message") {
	      SANE_Int v_i = 0;
	      char v_s[1024]; v_s[0] = 0;
	      
	      switch (opt->type)
		{
		case SANE_TYPE_BOOL:
                case SANE_TYPE_INT:
		  status = sane_control_option (device, i,
		                                SANE_ACTION_GET_VALUE,
                                                &v_i, 0);
		  break;
		case SANE_TYPE_FIXED:
		  break;
		case SANE_TYPE_STRING:
                  status = sane_control_option (device, i,
                                                SANE_ACTION_GET_VALUE,
                                                v_s, 0);
		  break;
		case SANE_TYPE_BUTTON:
		  break;
		case SANE_TYPE_GROUP:
		  break;
		}

              if (status != SANE_STATUS_GOOD) {
                std::cerr << "unable to obtain option value: "
                          << sane_strstatus (status) << std::endl;
		continue;
              }

	      if (opt->type == SANE_TYPE_BOOL && v_i) {
		std::cout << opt->title << std::endl;
		std::string s (opt->name);
		int i = s.find(' ');
		s.erase (0, i + 1);
		s.insert(0, "-");
		args.push_back (s);
		button_pressed = true;
	       }
	      else if (opt->type == SANE_TYPE_STRING && v_s[0] != 0 &&
		       button_pressed) {
		std::cout << "Message " << v_s << std::endl;
		args.push_back (v_s);
	      }
	    } // if button
	  } // if active
	} // for each option
      if (!button_pressed)
	usleep (250000);
    } // while (true)
  sane_close (device);

  if (script && button_pressed) {
    const char* c_args[10];
		const char** ca = c_args; *ca++ = script;
    for (std::vector<std::string>::iterator it = args.begin();
         it != args.end(); ++it)
      *ca++ = it->c_str();
    *ca = NULL;
    execv(script, (char* const*)c_args);
  }
  return 0;
}

